#!/usr/bin/env python
#
#   ConVirt   -  Copyright (c) 2008 Convirture Corp.
#   ======
#
# ConVirt is a Virtualization management tool with a graphical user
# interface that allows for performing the standard set of VM operations
# (start, stop, pause, kill, shutdown, reboot, snapshot, etc...). It
# also attempts to simplify various aspects of VM lifecycle management.
#
#
# This software is subject to the GNU General Public License, Version 2 (GPLv2)
# and for details, please consult it at:
#
#    http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
# 
#
# author : Jd <jd_jedi@users.sourceforge.net>
#

# Module containing classes for VM Settings and handlers

import re,pprint, traceback
import types
import os, sys,copy
import gtk, gtk.glade, gobject

from convirt.client.VMSettings import BootPage
from convirt.client.VMSettings import SettingsNotebook,VMNotebook
from convirt.client.VMSettings import VMSettingsDlg

from convirt.client.ui_utils import *

from convirt.client.XenUI import gladefile

class XenBootPage(BootPage):
    def __init__(self, context):
        BootPage.__init__(self, context)
        

    def ui_init(self, context):
        BootPage.ui_init(self, context)
        # construct and add required widgets.
        # and update the widgets list
        table = self.widgets.vm_boot_param_table
        rows = table.get_property("n-rows")
        cols = table.get_property("n-columns")
        
        # add 3 rows to the table and move exiting widgets down.
        table.resize(rows+3, cols)
        self.move_down(table, 3)

        # add the widgets
        xen_wtree =  gtk.glade.XML(gladefile, "boot_widget_container")

        # init_widget is additive : Watch for name collision
        self.init_widgets(xen_wtree, [ 
                                       "kernel_label",
                                       "kernel_entry",
                                       "kernel_selector_btn",
                                       "kernel_hbox",
                                       "ramdisk_label",
                                       "ramdisk_entry",
                                       "ramdisk_selector_btn",
                                       "ramdisk_hbox",
                                       "boot_loader_check_btn",
                                       "boot_loader_entry",
                                       "boot_widget_table"])


        self.dlg = xen_wtree.get_widget("boot_widget_container")
        self.dlg_table = self.widgets.boot_widget_table
        row = 0
        self.dlg_table.remove(self.widgets.kernel_label)
        self.dlg_table.remove(self.widgets.kernel_hbox)
        
        table.attach(self.widgets.kernel_label,0, 1,row, row+1)
        table.attach(self.widgets.kernel_hbox,1, 2,row, row+1)

        row +=1
        self.dlg_table.remove(self.widgets.ramdisk_label)
        self.dlg_table.remove(self.widgets.ramdisk_hbox)
        
        table.attach(self.widgets.ramdisk_label,0, 1,row, row+1)
        table.attach(self.widgets.ramdisk_hbox,1, 2,row, row+1)

        row += 1
        self.dlg_table.remove(self.widgets.boot_loader_check_btn)
        self.dlg_table.remove(self.widgets.boot_loader_entry)

        table.attach(self.widgets.boot_loader_check_btn,0, 1,row, row+1)
        table.attach(self.widgets.boot_loader_entry,1, 2,row, row+1)


        # setup handler
        xen_wtree.signal_connect("on_kernel_selector_btn_clicked",
                             self.on_kernel_selector_clicked)
        xen_wtree.signal_connect("on_ramdisk_selector_btn_clicked",
                             self.on_ramdisk_selector_clicked)
        xen_wtree.signal_connect("on_boot_loader_toggled",
                             self.on_boot_loader_toggled)


        self.dlg.hide()
        self.dlg.destroy()


        
    def on_kernel_selector_clicked(self, widget):
        file_selector(widget,self.context.managed_node,
                      field=self.widgets.kernel_entry,
                      fstype="open", parentwin=None)


    def on_ramdisk_selector_clicked(self, widget):
        file_selector(widget,self.context.managed_node,
                      field=self.widgets.ramdisk_entry,
                      fstype="open", parentwin=None)


    def on_boot_loader_toggled(self, widget):
        use_boot = widget.get_active()
        self.widgets.kernel_entry.set_sensitive(not use_boot)
        self.widgets.ramdisk_entry.set_sensitive(not use_boot)
        self.widgets.boot_loader_entry.set_sensitive(use_boot)

    def move_down(self, table, num_rows):
        for c in table.get_children():
            (left, right, top, bottom) =  \
                   table.child_get_property(c,"left-attach"),\
                   table.child_get_property(c,"right-attach"), \
                   table.child_get_property(c,"top-attach"),\
                   table.child_get_property(c,"bottom-attach")
            table.remove(c)
            table.attach(c, left, right, top + num_rows, bottom+num_rows)

    def get_widgets_to_disable(self, mode):
        base_widgets = BootPage.get_widgets_to_disable(self,mode)
        if mode in ["EDIT_VM_INFO",]:
            for widget in [ "kernel_entry",
                            "kernel_selector_btn",
                            "kernel_hbox",
                            "ramdisk_entry",
                            "ramdisk_selector_btn",
                            "ramdisk_hbox",
                            "boot_loader_check_btn",
                            "boot_loader_entry",
                             ]:
                base_widgets.append(self.widgets[widget])

        return base_widgets



    def scatter(self, context):
        BootPage.scatter(self,context)
        mode = context.mode
        vm_config = context.get("vm_config")
        if vm_config and mode != "EDIT_VM_INFO" :
            for widget, config_attr in (("kernel_entry", "kernel"),
                                        ("ramdisk_entry", "ramdisk"),
                                        ):
                set_value(self.widgets[widget], vm_config.get(config_attr))
                boot_loader = vm_config.get("bootloader")
                if boot_loader :
                    use_boot_loader = True
                else:
                    use_boot_loader = False
                self.widgets.kernel_entry.set_sensitive(not use_boot_loader)
                self.widgets.ramdisk_entry.set_sensitive(not use_boot_loader)
                self.widgets.boot_loader_entry.set_sensitive(use_boot_loader)
                set_value(self.widgets.boot_loader_check_btn, use_boot_loader)
                if boot_loader :
                    set_value(self.widgets.boot_loader_entry, boot_loader)
                else:
                    set_value(self.widgets.boot_loader_entry, '/usr/bin/pygrub')
        if vm_config and mode == "EDIT_VM_INFO":
            vm_info = self.context.vm_info
            if vm_info:
                for widget, config_attr in (("kernel_entry", "kernel"),
                                            ("ramdisk_entry", "ramdisk"),
                                            ("boot_loader_entry", "bootloader"),
                                            
                                        ):
                    set_value(self.widgets[widget], vm_info[config_attr])
                  
    def gather(self, context):
        BootPage.gather(self,context)
        mode = context.mode
        vm_config = context.get("vm_config")
        if vm_config:
            for widget, config_attr in (("kernel_entry", "kernel"),
                                        ("ramdisk_entry", "ramdisk"),
                                        ):
                vm_config[config_attr] = get_value(self.widgets[widget])

            use_boot_loader = get_value(self.widgets.boot_loader_check_btn)
            if use_boot_loader:
                vm_config["bootloader"] = get_value(self.widgets.boot_loader_entry)
                vm_config["kernel"] = ""
                vm_config["ramdisk"] = ""
            else:
                vm_config["bootloader"] = ""



class XenNotebook(VMNotebook):
    def __init__(self, context):
        VMNotebook.__init__(self, context)
        
    def get_boot_page(self):
        return XenBootPage(self.context)


class XenSettingsNotebook(SettingsNotebook):
    def __init__(self, context):
        SettingsNotebook.__init__(self, context)

    def get_vm_notebook_page(self):
        return XenNotebook(self.context)

class XenSettingsDlg(VMSettingsDlg):
    def __init__(self, mode,ctx, parent_win = None):
        VMSettingsDlg.__init__(self, mode,ctx, parent_win)

    def get_settings_notebook(self):
        return XenSettingsNotebook(self.context)









