#!/usr/bin/env python
#
#   ConVirt   -  Copyright (c) 2008 Convirture Corp.
#   ======
#
# ConVirt is a Virtualization management tool with a graphical user
# interface that allows for performing the standard set of VM operations
# (start, stop, pause, kill, shutdown, reboot, snapshot, etc...). It
# also attempts to simplify various aspects of VM lifecycle management.
#
#
# This software is subject to the GNU General Public License, Version 2 (GPLv2)
# and for details, please consult it at:
#
#    http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
# 
#
#

import types
import os, sys, re
import gtk, gtk.glade, gobject
import xmlrpclib
import convirt.core.utils.constants
from convirt.core.model.ManagedNode import ManagedNode, NodeException
from convirt.core.model.VM import VM
from convirt.core.utils.NodeProxy import Node
from convirt.core.utils.utils import XMConfig, is_host_remote, randomMAC, fetch_isp, show_url, get_path, guess_value
import urllib, urlparse
import xml.parsers.expat
import time
import traceback
from convirt.core.model.Groups import ServerGroup
from convirt.core.model.ImageStore import ImageStore
from convirt.client.htmltextview import HtmlTextView
import webbrowser

utils = convirt.core.utils.utils
constants = convirt.core.utils.constants


class AddServerGroupDialog:
    """ Class that handles adding new Server Pool"""

    initialized = False

    def __init__(self, wtree, left_nav):
        """ Constructor"""
        self.left_nav = left_nav

        self.dialog = wtree.get_widget("AddServerPool")
        self.group_name = wtree.get_widget("server_pool_name")

        self.group_name.set_activates_default(True)
        self.dialog.set_default_response(gtk.RESPONSE_OK)

        if not AddServerGroupDialog.initialized :
            # setup handlers
            wtree.signal_connect("on_add_server_pool_cancelbutton_clicked",
                                 self.on_cancel_button_clicked)
            wtree.signal_connect("on_add_server_pool_okbutton_clicked",
                                 self.on_ok_button_clicked)
            AddServerGroupDialog.initialized = True

        self.init()

    def init(self):
        self.group_name.set_text("")


    def show(self):
        """ Displays add server pool dialog"""

        ret = self.dialog.run()
        if ret == gtk.RESPONSE_DELETE_EVENT:
            self.dialog.hide()


    def on_cancel_button_clicked(self, widget):
       """  Cancel on add server pool dialog """
       self.init()
       self.dialog.hide()

    def on_ok_button_clicked(self, widget):
       """  Ok  button on add server pool """
       # validate parameters
       if self.group_name.get_text() == "":
           showmsg("Please enter valid group name")
           return

       try:
           group = ServerGroup(self.group_name.get_text())
           self.left_nav.add_group(group)
       except Exception , ex:
           showmsg(ex)
           return

       self.init()
       self.dialog.hide()


class CloneImageDialog:
    """ Class clone image """

    initialized = False

    def __init__(self, wtree, left_nav):
        """ Constructor"""
        self.left_nav = left_nav

        self.dialog = wtree.get_widget("CloneImage")
        self.img_name = wtree.get_widget("clone_image_name")
        self.selected_image_label  = wtree.get_widget("clone_selected_image")

        if not CloneImageDialog.initialized :
            # setup handlers
            wtree.signal_connect("on_clone_img_cancelbutton_clicked",
                             self.on_cancel_button_clicked)
            wtree.signal_connect("on_clone_img_okbutton_clicked",
                             self.on_ok_button_clicked)
            self.img_name.set_activates_default(True)
            self.dialog.set_default_response(gtk.RESPONSE_OK)
            CloneImageDialog.initialized = True

        self.init()

    def init(self):
        self.img_name.set_text("")


    def show(self, image_store, group_id, image_id):
        """ Displays clone image dialog"""
        self.image_store = image_store
        self.src_group_id = group_id
        self.src_image_id = image_id
        self.src_image_name = image_store.images[image_id].get_name()
        self.init()

        self.selected_image_label.set_text("Selected Image : " + self.src_image_name)
        ret = self.dialog.run()
        if ret == gtk.RESPONSE_DELETE_EVENT:
            self.dialog.hide()


    def on_cancel_button_clicked(self, widget):
       """  Cancel on clone image dialog """
       self.init()
       self.dialog.hide()

    def on_ok_button_clicked(self, widget):
       """  Ok  button on add server pool """
       # validate parameters
       img_name = self.img_name.get_text()
       if img_name == "":
           showmsg("Please enter valid image name")
           return
       img_name = img_name.strip()
       # may be we should change spaces to underscore ourselves.
# Image name can contain the special characters.       
#       if re.sub(ImageStore.INVALID_CHARS_EXP,"", img_name) != img_name:
#           showmsg("Image name can not contain any special chars %s" % \
#                              ImageStore.INVALID_CHARS)
#           return

       src_group = self.image_store.get_image_group(self.src_group_id)
       
       if src_group.image_exists_by_name(img_name):
           showmsg("Image already exists.")
           return

       self.image_store.clone_image(self.src_group_id, self.src_image_id,  img_name)
       self.left_nav.refresh_nodes()
       self.left_nav.set_selection("Image Store",
                                   self.left_nav.IMAGE_STORE,
                                   op="row-activated")
       self.init()
       self.dialog.hide()


class RenameImageDialog:
    """ Class rename image """

    initialized = False

    def __init__(self, wtree, left_nav):
        """ Constructor"""
        self.left_nav = left_nav

        self.dialog = wtree.get_widget("RenameImage")
        self.img_name = wtree.get_widget("image_name")
        self.selected_image_label  = wtree.get_widget("selected_image")

        if not RenameImageDialog.initialized :
            # setup handlers
            wtree.signal_connect("on_rename_img_cancelbutton_clicked",
                                 self.on_cancel_button_clicked)
            wtree.signal_connect("on_rename_img_okbutton_clicked",
                                 self.on_ok_button_clicked)
            self.img_name.set_activates_default(True)
            self.dialog.set_default_response(gtk.RESPONSE_OK)
            RenameImageDialog.initialized = True

        self.init()

    def init(self):
        self.img_name.set_text("")


    def show(self, image_store, group_id, image_id):
        """ Displays rename image dialog"""
        self.image_store = image_store
        self.src_group_id = group_id
        self.src_image_id = image_id
        self.src_image_name = image_store.images[image_id].get_name()
        self.init()

        self.selected_image_label.set_text("Selected Image : " + self.src_image_name)
        ret = self.dialog.run()
        if ret == gtk.RESPONSE_DELETE_EVENT:
            self.dialog.hide()


    def on_cancel_button_clicked(self, widget):
       """  Cancel on rename image dialog """
       self.init()
       self.dialog.hide()

    def on_ok_button_clicked(self, widget):
       """  Ok  button on rename image """
       # validate parameters
       img_name = self.img_name.get_text()
       if img_name == "":
           showmsg("Please enter valid image name")
           return
       img_name = img_name.strip()
       # may be we should change spaces to underscore ourselves.
# Image name can contain the special characters.       
#       if re.sub(ImageStore.INVALID_CHARS_EXP,"", img_name) != img_name:
#           showmsg("Image name can not contain any special chars %s" % \
#                              ImageStore.INVALID_CHARS)
#           return
       if img_name in self.image_store.images:
           showmsg("Image already exists.")
           return

       self.image_store.rename_image(self.src_group_id, self.src_image_id,  img_name)
       self.left_nav.refresh_nodes()
       self.left_nav.set_selection("Image Store",
                                   self.left_nav.IMAGE_STORE,
                                   op="row-activated")
       self.init()
       self.dialog.hide()


class RenameImageGroupDialog:
    """ Class rename image group"""

    initialized = False

    def __init__(self, wtree, left_nav):
        """ Constructor"""
        self.left_nav = left_nav

        self.dialog = wtree.get_widget("RenameImageGroup")
        self.img_group_name = wtree.get_widget("img_group_name")
        self.selected_image_group_label  = wtree.get_widget("selected_image_group")

        if not RenameImageGroupDialog.initialized :
            # setup handlers
            wtree.signal_connect("on_rename_img_grp_cancelbutton_clicked",
                                 self.on_cancel_button_clicked)
            wtree.signal_connect("on_rename_img_grp_okbutton_clicked",
                                 self.on_ok_button_clicked)
            self.img_group_name.set_activates_default(True)
            self.dialog.set_default_response(gtk.RESPONSE_OK)
            RenameImageGroupDialog.initialized = True

        self.init()

    def init(self):
        self.img_group_name.set_text("")


    def show(self, image_store, group_id):
        """ Displays rename image group dialog"""
        self.image_store = image_store
        self.src_group_id = group_id

        self.src_image_name = image_store.image_groups[group_id].get_name()
        self.init()

        self.selected_image_group_label.set_text("Selected Image Group: " + self.src_image_name)
        ret = self.dialog.run()
        if ret == gtk.RESPONSE_DELETE_EVENT:
            self.dialog.hide()


    def on_cancel_button_clicked(self, widget):
       """  Cancel on rename image group dialog """
       self.init()
       self.dialog.hide()

    def on_ok_button_clicked(self, widget):
       """  Ok  button on rename image group"""
       # validate parameters
       img_name = self.img_group_name.get_text()
       if img_name == "":
           showmsg("Please enter valid image name")
           return
       img_name = img_name.strip()
       # may be we should change spaces to underscore ourselves.
# Image name can contain the special characters.       
#       if re.sub(ImageStore.INVALID_CHARS_EXP,"", img_name) != img_name:
#           showmsg("Image name can not contain any special chars %s" % \
#                              ImageStore.INVALID_CHARS)
#           return
       if img_name in self.image_store.image_groups:
           showmsg("Image already exists.")
           return

       self.image_store.rename_image_group(self.src_group_id,  img_name)
       self.left_nav.refresh_nodes()
       self.left_nav.set_selection("Image Store",
                                   self.left_nav.IMAGE_STORE,
                                   op="row-activated")
       self.init()
       self.dialog.hide()

class AddImageGroupDialog:
    """ Class that handles adding new Image Group"""

    initialized = False

    def __init__(self, wtree, left_nav):
        """ Constructor"""
        self.left_nav = left_nav

        self.dialog = wtree.get_widget("AddImageGroup")
        self.group_name = wtree.get_widget("image_group_name")

        self.group_name.set_activates_default(True)
        self.dialog.set_default_response(gtk.RESPONSE_OK)

        if not AddImageGroupDialog.initialized :
            # setup handlers
            wtree.signal_connect("on_add_image_group_cancelbutton_clicked",
                                 self.on_cancel_button_clicked)
            wtree.signal_connect("on_add_image_group_okbutton_clicked",
                                 self.on_ok_button_clicked)
            AddImageGroupDialog.initialized = True

        self.init()

    def init(self):
        self.group_name.set_text("")


    def show(self, image_store):
        """ Displays add image group dialog"""
        self.image_store = image_store
        ret = self.dialog.run()
        if ret == gtk.RESPONSE_DELETE_EVENT:
            self.dialog.hide()


    def on_cancel_button_clicked(self, widget):
       """  Cancel on add image group dialog """
       self.init()
       self.dialog.hide()

    def on_ok_button_clicked(self, widget):
       """  Ok  button on add image group """
       # validate parameters
       if self.group_name.get_text() == "":
           showmsg("Please enter valid group name") 
           return

       try:
           group = self.image_store.new_group(self.group_name.get_text()) 
           self.image_store.add_group(group)
           self.left_nav.refresh_nodes()
           self.left_nav.set_selection("Image Store",
                                   self.left_nav.IMAGE_STORE,
                                   op="row-activated")
       except Exception , ex:
           showmsg(ex)
           return

       self.init()
       self.dialog.hide()



class Cred:
    def __init__(self, username, password):
        self.username = username
        self.password = password

    def get_username(self):
        return self.username

    def get_password(self):
        return self.password


class CredentialsHelper:

    def __init__(self,wtree):
        self.wtree = wtree


    def get_credentials(self, hostname, username=None):
        dlg = CredentialsDialog(self.wtree)
        dlg.show(hostname, username)
        ret = dlg.dialog.run()
        cred = None
        if ret == gtk.RESPONSE_OK:
            cred = Cred(dlg.username.get_text(), dlg.password.get_text())
        dlg.init()
        return cred

class CredentialsDialog:
    """ Class that handles events from collecting credentials"""
    initialized = False
    def __init__(self, wtree):
        """ Constructor"""
        self.dialog = wtree.get_widget("Credentials")
        self.username = wtree.get_widget("cred_username")
        self.password = wtree.get_widget("cred_password")
        self.cred_label = wtree.get_widget("cred_label")

        self.password.set_activates_default(True)
        self.dialog.set_default_response(gtk.RESPONSE_OK)

        if not CredentialsDialog.initialized:
            # setup handlers
            wtree.signal_connect("on_cred_cancelbutton_clicked",
                                 self.on_cancel_button_clicked)
            wtree.signal_connect("on_cred_okbutton_clicked",
                                 self.on_ok_button_clicked)
            CredentialsDialog.initialized = True
        self.init()

    def init(self):
        self.username.set_text("")
        self.password.set_text("")

    def show(self, hostname, username = None):
        """ Displays initialization params dialog"""
        if username is not None:
            self.username.set_text(username)
            self.password.grab_focus()
        self.cred_label.set_text("Enter Credentials : " + hostname)
        #self.dialog.set_title("Credentials for " + hostname)
        self.dialog.show()

    def on_cancel_button_clicked(self, widget):
       """  Cancel on initialization params """
       self.dialog.hide()

    def on_ok_button_clicked(self, widget):
       """  Cancel creation of dom """
       self.dialog.hide()



class RemoteFileDialog:
    """ Simple dialog box to take file or path from the user """
    initialized = False
    def __init__(self, wtree):
        """ Constructor"""
        self.dialog = wtree.get_widget("RemoteFileDialog")
        self.filename = wtree.get_widget("remote_file_filename")
        self.file_selector = wtree.get_widget("remote_file_selector")
        self.file_label = wtree.get_widget("remote_file_label")
        self.hostname = wtree.get_widget("remote_file_hostname")
        self.frame_label = wtree.get_widget("remote_file_frame_title")
        if not RemoteFileDialog.initialized:
            # setup handlers
            wtree.signal_connect("on_remote_file_cancel_button_clicked",
                                 self.on_cancel_button_clicked)
            wtree.signal_connect("on_remote_file_okbutton_clicked",
                                 self.on_ok_button_clicked)
            wtree.signal_connect("on_remote_file_selectorButton_clicked",
                                 self.on_file_selector_clicked)
            RemoteFileDialog.initialized = True

        self.init()

    def init(self):
        self.filename.set_text("")

    def show(self, managed_node,
             title, fstype, init_folder,
             init_file, parentwin, multi_select=False):

        self.managed_node = managed_node
        self.title = title
        self.fstype = fstype
        self.init_folder = init_folder
        self.init_file = init_file
        self.parentwin = parentwin

        self.hostname.set_text(managed_node.hostname)

        if self.fstype == "select_folder":
            self.file_label.set_text("Folder name")
            self.frame_label.set_markup("<b>Specify Folder </b>")
        else:
            self.file_label.set_text("Filename")
            if multi_select:
                self.frame_label.set_markup("<b>Specify Filenames (comma separated) </b>")
            else:
                self.frame_label.set_markup("<b>Specify Filename </b>")

        self.dialog.set_title(title)

        # choose decent default
        if init_folder is not None and init_file is not None:
            # remove trailing / from path
            if init_folder.rfind("/") == len(init_folder) - 1:
                init_folder = init_folder[0:len(init_folder) -1]
            # remove leading / from filename
            if init_file.find("/") == 0:
                init_file = init_file[1:]
            self.filename.set_text(init_folder + "/" + init_file)

        elif init_folder is not None :
            self.filename.set_text(init_folder)
        elif init_file is not None:
            self.filename.set_text(init_file)

    def on_cancel_button_clicked(self, widget):
       """  Cancel on RemoteFileDialog  """
       self.dialog.hide()

    def on_ok_button_clicked(self, widget):
       """  Ok on RemoteFileDialog """
       self.dialog.hide()

    def on_file_selector_clicked(self, widget):
       """  file selector clicked on RemoteFileDialog """
       # force the file selection to local/mounted files.
       (res, selection) = file_selection(self.managed_node,
                                         self.title,
                                         self.fstype,
                                         self.init_folder, self.init_file,
                                         self.parentwin,
                                         force_local = True,
                                         multi_select = True)

       if res and selection:
           text = ""
           for file in selection:
               text= text + file + ","

           if len(text) > 0 and text[-1] == ",":
               text = text[0:-1]

           self.filename.set_text(text)

    def run(self):
        return self.dialog.run()

    def get_filename(self):
        return self.filename.get_text()


class FileViewEditDialog:
    """ Allow a text file to be viewed/edited """
    initialized = False
    def __init__(self, wtree):
        """ Constructor"""
        self.dialog = wtree.get_widget("FileViewEditDialog")
        self.save_button = wtree.get_widget("file_view_edit_save")
        self.cancel_button = wtree.get_widget("file_view_edit_cancel")
        self.file_content = wtree.get_widget("file_content")
        if not FileViewEditDialog.initialized:
            # setup handlers
            wtree.signal_connect("on_file_view_edit_save_clicked",
                                 self.on_save_button_clicked)
            wtree.signal_connect("on_file_view_edit_cancel_clicked",
                                 self.on_cancel_button_clicked)
            FileViewEditDialog.initialized = True

        self.managed_node = None
        self.filename = None
        self.parentwin = None
        self.editable = True

    #    self.init()

    #def init(self):
    #    self.file_content.set_text("")

    def show(self, widget,  managed_node, filename, content = None,editable=True,
             parentwin = None):

        self.managed_node = managed_node
        self.filename = filename
        self.parentwin = parentwin
        self.editable = editable
        if content is not None:
            self.text = content
            self.editable = False
        else:
            if managed_node is not None and filename is not None:
                self.text = self.get_text_from_file(self.managed_node,
                                                    self.filename)
            else:
                self.text = ""

        self.text_buffer = gtk.TextBuffer()
        self.text_buffer.set_text(self.text)
        self.file_content.set_buffer(self.text_buffer)
        if self.managed_node is not None and self.filename is not None:
            self.dialog.set_title("File : "+  \
                                  self.managed_node.hostname + ":" + self.filename)
        else:
            self.dialog.set_title("Content")

        # adjusts the buttons
        if not self.editable:
            self.save_button.set_label("Ok")
        else:
            self.save_button.set_label("Save")

        self.cancel_button.set_property("visible", self.editable)

        if parentwin:
            self.dialog.set_transient_for(parentwin)
            self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)


        #self.dialog.show()
        ret = self.dialog.run()
        if ret == gtk.RESPONSE_DELETE_EVENT:
            self.dialog.hide()


    def on_save_button_clicked(self, widget):
        """  Save on FileViewEditDialog  """
        if self.editable:
            buf = self.file_content.get_buffer()
            self.text = buf.get_text(buf.get_start_iter(),
                                     buf.get_end_iter())
            print "save to text  " , self.filename
            self.save_text_to_file(self.managed_node,self.filename, self.text)

        self.dialog.hide()

    def on_cancel_button_clicked(self, widget):
        """  Cancel on FileViewEditDialog """
        self.dialog.hide()

    def get_text_from_file(self, managed_node,filename):
        text=""
        file = managed_node.node_proxy.open(filename)
        lines = file.readlines()
        text = "".join(lines)
        file.close()
        return text

    def save_text_to_file(self, managed_node,filename, text):
        file = managed_node.node_proxy.open(filename, "w")
        file.write(text)
        file.close()


class NodeSelection:
    """ Show dialog with list of nodes and return the selection """
    initialized = False
    def __init__(self, wtree):
        """ Constructor"""

        self.dialog = wtree.get_widget("NodeSelection")
        self.node_list_view = wtree.get_widget("select_node_list")
        self.migrate_live_check_button = wtree.get_widget("migrate_live")
        self.select_node_label = wtree.get_widget("select_node_label")
        #self.node_list_view.set_activates_default(True)
        self.dialog.set_default_response(gtk.RESPONSE_OK)
        self.list_model = None
        self.selected_node_name = None
        self.selected_node = None
        self.selected_node_type = None
        self.allow_groups = False
        self.migration_mode = False
        self.migrate_live_selection = False
        self.selected_group_name = None

        pbrenderer = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn("       ", pbrenderer)
        column.set_cell_data_func(pbrenderer, get_state_pixbuf)
        self.node_list_view.append_column(column)

        textrenderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Name", textrenderer, text=0)

        self.node_list_view.append_column(column)

        self.node_list_model = gtk.TreeStore(gobject.TYPE_STRING,
                                             gobject.TYPE_STRING,
                                             gobject.TYPE_PYOBJECT,
                                             gobject.TYPE_STRING,
                                             gobject.TYPE_STRING,
                                             gobject.TYPE_STRING)

        self.node_list_view.set_model(self.node_list_model)

        if not NodeSelection.initialized:
                        # setup handlers
            wtree.signal_connect("on_node_selection_ok_clicked",
                                 self.on_ok_button_clicked)
            wtree.signal_connect("on_node_selection_cancel_clicked",
                                 self.on_cancel_button_clicked)
            NodeSelection.initialized = True

    def get_selection(self):
        return (self.selected_node_name,
                self.selected_node_type,
                self.selected_node,
                self.selected_group_name,
                self.migrate_live_selection)


    def show(self, widget, manager, migration = False, allow_groups =False,
             parentwin = None, platform = None):

        self.parentwin = parentwin
        self.selected_node_name = None
        self.selected_node = None
        self.selected_node_type = None
        self.migrate_live_selection = False
        self.migration_mode = migration
        self.allow_groups = allow_groups

        if allow_groups:
            self.select_node_label.set_text("Select a Managed Server or a Server Pool")
        else:
            self.select_node_label.set_text("Select a Managed Server")

        populate_nodes(manager, self.node_list_view, self.node_list_model,
                       False, platform)

        self.migrate_live_check_button.set_property("visible",
                                                    self.migration_mode)
        self.migrate_live_check_button.set_active(self.migration_mode)

        self.dialog.show()
        ret = self.dialog.run()
        if ret == gtk.RESPONSE_DELETE_EVENT:
            self.dialog.hide()


    def on_ok_button_clicked(self, widget):
        """  Ok on Select Node  """
        widget = self.node_list_view.get_selection()
        treemodel, treeiter = widget.get_selected()
        if treeiter:
            (name,type,node,group_name)= \
                                         (treemodel.get_value(treeiter, 0),
                                          treemodel.get_value(treeiter, 1),
                                          treemodel.get_value(treeiter, 2),
                                          treemodel.get_value(treeiter, 3))


            if type is not None and type == MANAGED_NODE or \
               (type is not None and type == SERVER_POOL and
                self.allow_groups == True):
                self.selected_node_name = name
                self.selected_node = node
                self.selected_node_type = type
                self.selected_group_name = group_name
                if self.migration_mode :
                    self.migrate_live_selection = self.migrate_live_check_button.get_active()
            else:
                showmsg("Invalid Selection.")
                return
        self.dialog.hide()

    def on_cancel_button_clicked(self, widget):
        """  Cancel on FileViewEditDialog """
        self.dialog.hide()



# Make this non-modal as now can be displayed from multiple
# provisioning dialogs.

class ProvStatusDlg:
    """Displays the status message and output and logfile"""
    def __init__(self):
        """ Constructor"""
        self.gladefile = gladefile
        self.wtree = gtk.glade.XML(self.gladefile, "ProvStatusDlg")
        self.dialog = self.wtree.get_widget("ProvStatusDlg")
        self.msg_widget = self.wtree.get_widget("msg_text")
        self.output = ""
        self.log_filename = None

        # setup handlers
        self.wtree.signal_connect("on_msg_close_clicked",
                                  self.on_close_button_clicked)
        self.wtree.signal_connect("on_msg_view_output_clicked",
                                  self.on_output_button_clicked)
        self.wtree.signal_connect("on_msg_view_log_clicked",
                                  self.on_log_button_clicked)

    def show(self, widget, msg, managed_node, log_filename, output,
             parentwin = None):

        self.managed_node = managed_node
        self.parentwin = parentwin
        self.msg = msg
        self.output = output
        self.log_filename = log_filename

        self.msg_widget.set_text(msg)

        if parentwin:
            self.dialog.set_transient_for(parentwin)
            self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)


        self.dialog.show()
        # Non-modal op now.
        #ret = self.dialog.run()
        #if ret == gtk.RESPONSE_DELETE_EVENT:
        #    self.dialog.hide()


    def on_output_button_clicked(self, widget):
        """  output button on ProvStatusDlg   """
        dlg = file_editor.show(widget,
                               None,
                               None,
                               content = self.output,
                               editable = False,
                               parentwin = self.dialog)

    def on_log_button_clicked(self, widget):
        """  output button on ProvStatusDlg   """
        dlg = file_editor.show(widget,
                               self.managed_node,
                               self.log_filename,
                               editable=False,
                               parentwin = self.dialog)



    def on_close_button_clicked(self, widget):
        """  Cancel on ProvStatusDlg """
        self.dialog.destroy()


class GetLocalIPDlg:

    def __init__(self):

        """ Constructor"""
        self.gladefile = gladefile
        self.wtree = gtk.glade.XML(self.gladefile, "local_ip_dlg")
        
        self.dialog = self.wtree.get_widget("local_ip_dlg")

        self.local_ip_entry = self.wtree.get_widget("local_ip_entry")

        self.dialog.set_default_response(gtk.RESPONSE_OK)
        # connect the handlers
        self.wtree.signal_connect("on_local_ip_cancel_clicked",
                                 self.on_cancel_clicked)
        self.wtree.signal_connect("on_local_ip_ok_clicked",
                             self.on_ok_clicked)


    def show(self, manager, local_node, parentwin = None):

        self.manager = manager
        self.local_node = local_node

        self.parentwin = parentwin

        if parentwin:
            self.dialog.set_transient_for(parentwin)
            self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

        adr = local_node.get_address()
        if adr and adr.lower() != "localhost":
            self.local_ip_entry.set_text(adr)
        else:
            self.local_ip_entry.set_text("")

        self.dialog.show()
        ret = self.dialog.run()

        self.dialog.destroy()
        return ret

    def on_cancel_clicked(self, widget):
        return False



    def on_ok_clicked(self, widget):
        local_ip = self.local_ip_entry.get_text()
        local_ip = local_ip.strip()
        if not local_ip:
            showmsg("Please enter a hostname of IP address.")
            return
        self.local_node.set_address(local_ip)
        self.manager._save_node(self.local_node)
        return True




class MigrationChecksResultsDlg:

    (TYPE,CATEGORY,DETAILS) = range(3)

    def __init__(self):

        self.gladefile = gladefile
        self.wtree = gtk.glade.XML(self.gladefile, "MigrationResultDlg")
        self.dialog = self.wtree.get_widget("MigrationResultDlg")

        """ Constructor"""
        # the list view
        self.list_view = self.wtree.get_widget("m_chk_tree_view")

        # set default response
        self.dialog.set_default_response(gtk.RESPONSE_OK)

        # state variables
        self.list_model = None


        # lets populate the colums

        # ENABLE THIS FOR Warning/Error Icon
        pbrenderer = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn("Type", pbrenderer)
        column.set_cell_data_func(pbrenderer, self.get_msg_type_pb)
        column.set_sort_column_id(self.TYPE)
        self.list_view.append_column(column)

        lt_textrenderer = gtk.CellRendererText()
        lt_textrenderer.set_property("xalign", 0.0)

        ## column = gtk.TreeViewColumn("Type", lt_textrenderer, text=0)
##         column.set_clickable(True)
##         column.set_sort_column_id(self.TYPE)
##         column.set_resizable(True)
##         self.list_view.append_column(column)


        textrenderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Category", textrenderer, text=1)
        column.set_clickable(True)
        column.set_sort_column_id(self.CATEGORY)
        column.set_resizable(True)
        self.list_view.append_column(column)

        textrenderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Message", textrenderer, text=2)
        column.set_clickable(True)
        column.set_sort_column_id(self.DETAILS)
        column.set_resizable(True)
        self.list_view.append_column(column)


        self.list_model = gtk.TreeStore(gobject.TYPE_STRING, # Type
                                        gobject.TYPE_STRING, # Category
                                        gobject.TYPE_STRING) # Message



        self.list_view.set_model(self.list_model)

        self.list_model.set_sort_column_id(self.TYPE, gtk.SORT_ASCENDING)


    def populate_list(self, err_list, warn_list):
        for err in err_list:
            (cat, msg) = err
            iter = self.list_model.insert_before(None, None)
            self.list_model.set(iter,
                                self.TYPE,"Error",
                                self.CATEGORY,cat,
                                self.DETAILS,msg)

        for warn in warn_list:
            (cat, msg) = warn
            iter = self.list_model.insert_before(None, None)
            self.list_model.set(iter,
                                self.TYPE,"Warning",
                                self.CATEGORY,cat,
                                self.DETAILS,msg)


    def get_msg_type_pb(self,column,cell,model,iter):
        type = model.get_value(iter, self.TYPE)
        id = None
        if type:
            if type == "Warning":
                id = gtk.STOCK_DIALOG_WARNING
            elif type == "Error":
                id = gtk.STOCK_DIALOG_ERROR

        cell.set_property('stock-id', id)

    def show(self, err_list, warn_list, parentwin=None):

        self.err_list = err_list
        self.warn_list = warn_list
        self.parentwin = parentwin

        if parentwin:
            self.dialog.set_transient_for(parentwin)
            self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

        self.populate_list(self.err_list, self.warn_list)

        self.dialog.show()
        ret = self.dialog.run()
        self.dialog.destroy()

        if ret == gtk.RESPONSE_OK:
            return True
        else:
            return False

class ApplianceDetailsDlg:

    def __init__(self):

        """ Constructor"""
        self.gladefile = gladefile
        self.wtree = gtk.glade.XML(self.gladefile, "appliance_details_dlg")
        self.dialog = self.wtree.get_widget("appliance_details_dlg")

        self.w_app_protocol = self.wtree.get_widget("app_protocol")
        self.w_app_hostname = self.wtree.get_widget("app_hostname")
        self.w_app_port = self.wtree.get_widget("app_port")
        self.w_app_path = self.wtree.get_widget("app_path")

        self.w_app_mgmt_protocol = self.wtree.get_widget("app_mgmt_protocol")
        self.w_app_mgmt_port = self.wtree.get_widget("app_mgmt_port")

        self.dialog.set_default_response(gtk.RESPONSE_OK)
        # connect the handlers
        self.wtree.signal_connect("on_app_details_cancel_clicked",
                                 self.on_cancel_clicked)
        self.wtree.signal_connect("on_app_details_ok_clicked",
                             self.on_ok_clicked)

        self.mapping = ((self.w_app_protocol, "app_protocol",
                         "default_app_protocol"),
                        (self.w_app_hostname, "host",
                         "default_app_hostname"),
                        (self.w_app_port, "app_port", "default_app_port"),
                        (self.w_app_path, "app_path", "default_app_path"),
                        (self.w_app_mgmt_protocol, "app_mgmt_protocol",
                         "default_app_mgmt_protocol"),
                        (self.w_app_mgmt_port, "app_mgmt_port", "default_app_mgmt_port"))

        self.protocols = ["http", "https", "ssh_tunnel"]

    def show(self, vm_config, defaults={}, parentwin = None):

        self.vm_config = vm_config
        self.defaults = defaults

        self.parentwin = parentwin

        if parentwin:
            self.dialog.set_transient_for(parentwin)
            self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)



        # populate
        for w, vkey, dkey in self.mapping:
            val = self.vm_config.get(vkey)
            if not val:
                val = self.defaults.get(dkey)

            if not val:
                val = ""

            if isinstance(w, gtk.Entry):
                w.set_text(str(val))
            elif isinstance(w, gtk.ComboBox):
                print val
                if val in self.protocols:
                    w.set_active(self.protocols.index(val))
                else:
                    w.set_active(-1)

        self.dialog.show()
        ret = self.dialog.run()

        if ret == gtk.RESPONSE_OK:
            for w, vkey, dkey in self.mapping:
                if isinstance(w, gtk.Entry):
                    val = w.get_text()
                    if val and vkey in ("app_port", "app_mgmt_port"):
                        val = int(val)

                elif isinstance(w, gtk.ComboBox):
                    val = self.protocols[w.get_active()]
                self.vm_config[vkey] = val

            self.vm_config.write()
            self.dialog.destroy()
            return True
        else:
            self.dialog.destroy()
            return False

    def on_cancel_clicked(self, widget):
        return False


    def on_ok_clicked(self, widget):
        return True

class GroupVariablesDlg:
    """ Show dialog with list of group variables and return the selection """
    (COL_ATTR, COL_VAL, COL_EDITABLE) = range(3)
    def __init__(self, group_vars = None):
        """ Constructor"""
        print gladefile
        self.gladefile = gladefile
        self.wtree = gtk.glade.XML(self.gladefile, "GroupVariablesDlg")
        self.dialog = self.wtree.get_widget("GroupVariablesDlg")

        self.groupvar_list_view = self.wtree.get_widget("select_groupvar_list")
        self.dialog.set_default_response(gtk.RESPONSE_OK)
        self.groupvar_list_model  = gtk.ListStore( gobject.TYPE_STRING,
                                                   gobject.TYPE_STRING,
                                                   gobject.TYPE_BOOLEAN)

        self.groupvar_list_view.set_model(self.groupvar_list_model)
        self.groupvar_list_view.set_rules_hint(True)
        self.groupvar_list_view.get_selection().set_mode(gtk.SELECTION_SINGLE)

        self.groupvar_selected = {}


        renderer = gtk.CellRendererText()
        renderer.connect("edited", self.__on_props_cell_edited,
                         self.groupvar_list_model)
        renderer.connect("editing-started",
                         self.__on_props_cell_start_editing,
                         self.groupvar_list_model)


        
        renderer.set_data("column", self.COL_ATTR)

        column = gtk.TreeViewColumn("Variable", renderer, text=self.COL_ATTR,
                                    editable=self.COL_EDITABLE)

        self.groupvar_list_view.append_column(column)

        renderer = gtk.CellRendererText()
        renderer.connect("edited",
                         self.__on_props_cell_edited,
                         self.groupvar_list_model)
        renderer.connect("editing-started",
                         self.__on_props_cell_start_editing,
                         self.groupvar_list_model)


        renderer.set_data("column", self.COL_VAL)

        column = gtk.TreeViewColumn("Value", renderer, text=self.COL_VAL,
                                    editable=self.COL_EDITABLE)
        self.groupvar_list_view.append_column(column)

        self.wtree.signal_autoconnect(
            {
            "on_groupvar_ok_clicked": self.on_ok_button_clicked,
            "on_groupvar_cancel_clicked": self.on_cancel_button_clicked,
            "on_groupvar_add_clicked": ( self.on_add_button_clicked,
                                         self.groupvar_list_model,
                                         self.groupvar_list_view ),
            "on_groupvar_remove_clicked": self.on_remove_button_clicked,
             })

        #If group variables are provided then set them in the model.
        if group_vars is not None:
            self.set_group_vars(group_vars)

    # hook to catch the context of the cell and set signal handlers for
    # the gtk.Entry.
    def __on_props_cell_start_editing(self,cell,editable, path_string, model):
        self.escape = False
        iter = model.get_iter_from_string(path_string)
        path = model.get_path(iter)[0]
        column = cell.get_data("column")
        editable.connect("focus-out-event", self.__on_focus_out,
                         (path_string,column,model))
        editable.connect("key-press-event", self.__on_key_released)

    # Take care of the ESCAPE key. set the flag and return False for the
    # event system to continue processing.
    def __on_key_released(self, widget, event):
        self.escape =  (event.keyval == 65307) # escape key
        return False


    def __on_focus_out(self,editable, event, context):
        if self.escape:
            return
        (path_string, column, model) = context
        iter = model.get_iter_from_string(path_string)
        new_text = editable.get_text()
        model.set(iter, column, new_text)

    def __on_props_cell_edited(self,cell,path_string,new_text, model):
        iter = model.get_iter_from_string(path_string)
        path = model.get_path(iter)[0]
        column = cell.get_data("column")
        model.set(iter, column, new_text)

    def on_cancel_button_clicked(self, widget):
        self.groupvar_selected = None
        self.dialog.destroy()
        return False

    def get_group_vars(self):
        return (self.groupvar_selected)

    def set_group_vars(self, groupvars):
        model = self.groupvar_list_model
        for (k,v) in groupvars.iteritems():
            iter = model.append()
            model.set(iter,
                  self.COL_ATTR, k,
                  self.COL_VAL, v,
                  self.COL_EDITABLE, True)


    def on_ok_button_clicked(self, widget):
        print "Ok button"
        model = self.groupvar_list_model
        self.groupvar_selected = {}
        iter = model.get_iter_first()
        while iter:
            key = model.get_value(iter, self.COL_ATTR)
            val = model.get_value(iter, self.COL_VAL)
            self.groupvar_selected[key] = val
            iter = model.iter_next(iter)

        for (k,v) in self.groupvar_selected.iteritems():
            print k, v
        self.dialog.destroy()
        return True

    def on_add_button_clicked(self, widget, model, treeview):
        print "Add button"
        iter = model.append()
        model.set(iter,
                  self.COL_ATTR, "",
                  self.COL_VAL, "",
                  self.COL_EDITABLE, True)

        # set the higlight to newly added row.
        path = model.get_path(iter)
        treeview.set_cursor(path,treeview.get_column(0),True)


    def on_remove_button_clicked(self, widget):
        print "Remove button"
        selection = self.groupvar_list_view.get_selection()
        if selection:
            model, iter = selection.get_selected()
            if iter:
                path = model.get_path(iter)[0]
                model.remove(iter)


    def show(self, widget, pool, parentwin = None):
        """ Displays add group variables dialog"""
        ret = self.dialog.run()


# show the news / updates for the product
class UpdatesDlg:

    def __init__(self):

        """ Constructor"""
        self.gladefile = gladefile
        self.wtree = gtk.glade.XML(self.gladefile, "updates_dlg")
        self.dialog = self.wtree.get_widget("updates_dlg")
        self.view_wnd = self.wtree.get_widget("updates_dlg_view_wnd")

        self.view = self.wtree.get_widget("update_view")
        self.view_wnd.remove(self.view)
        self.view = HtmlTextView()

        self.view.set_property("visible",True)
        self.view.set_wrap_mode(gtk.WRAP_WORD)
        self.view.set_editable(False)

        self.view.connect("url-clicked", self.url_clicked_cb)
        self.view_wnd.add(self.view)

        self.title_span = '<span style="font-size: 125%; font-family: serif; color:#0000FF;text-align: left">'

        # connect the handlers
        self.wtree.signal_connect("on_updates_dlg_close_clicked",
                                 self.on_close_button_clicked)


    def show(self, updates, widget = None,parentwin = None):
        self.updates = updates
        self.parentwin = parentwin

        if parentwin:
            self.dialog.set_transient_for(parentwin)
            self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

        text = "<body>"

        for update in updates:
            text += self.title_span
            text += '<a href="' + str(update["link"]) + '">'
            text = text + str(update["title"])
            text += "</a>"
            text += "</span>"
            text += "<br/>"
            text += "<br/>"

            text = text + str(update["description"])
            text += "<br/>"
            text += "<br/>"


        text += "</body>"
        try:
            self.view.display_html(text)
        except Exception, ex:
            print "error showing ConVirt updates : HTML -> " + text + "\n" + \
                  "Exception " + str(ex)

        self.dialog.show()

    def on_close_button_clicked(self, widget):
        self.dialog.destroy()

    def url_clicked_cb(htmlview, self, url, type):
        show_url(url)

# show the total number of VM's being managed
class LicenseDlg:
    (SERVER_GROUP,SERVER,VMS, DUMMY) = range(4)
    def __init__(self, store):

        """ Constructor"""
        self.store = store
        self.gladefile = gladefile
        self.wtree = gtk.glade.XML(self.gladefile, "show_license_dlg")
        self.dialog = self.wtree.get_widget("show_license_dlg")
        self.view_wnd = self.wtree.get_widget("license_dlg_view_wnd")

        self.view = self.wtree.get_widget("license_view")


        self.view.set_property("visible",True)

        # connect the handlers
        self.wtree.signal_connect("on_license_dlg_close_clicked",
                                  self.on_close_button_clicked)

        textrenderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Server Group", textrenderer, text=0)
        column.set_resizable(True)
        self.view.append_column(column)

        textrenderer = gtk.CellRendererText()
        textrenderer.set_property("xalign", 0.5)
        column = gtk.TreeViewColumn("Servers", textrenderer, text=1)
        column.set_resizable(True)
        self.view.append_column(column)

        textrenderer = gtk.CellRendererText()
        textrenderer.set_property("xalign", 0.5)
        column = gtk.TreeViewColumn("VMs", textrenderer, text=2)
        column.set_resizable(True)
        self.view.append_column(column)

        textrenderer = gtk.CellRendererText()
        #textrenderer.set_property("xalign", 0.5)
        column = gtk.TreeViewColumn("", textrenderer, text=3)
        column.set_resizable(True)
        self.view.append_column(column)


        self.list_model = gtk.TreeStore(gobject.TYPE_STRING,
                                        gobject.TYPE_STRING,
                                        gobject.TYPE_STRING,
                                        gobject.TYPE_STRING)



        self.view.set_model(self.list_model)


    def show(self, manager, widget = None,parentwin = None ):
        self.parentwin = parentwin
        self.manager = manager
        if parentwin:
            self.dialog.set_transient_for(parentwin)
            self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

        self.populate_list(manager)

        self.dialog.show()
        ret = self.dialog.run()
        self.dialog.destroy()


    def on_close_button_clicked(self, widget):
        self.dialog.destroy()

    def populate_list(self, manager):
        #Iterate through uncategorized nodes and print the vm count
        nodes = manager.getNodeList()
        vmCount = 0
        totalServerCount=0
        totalVmCount = 0
        
        for i in nodes.keys():
          vmCount += nodes[i].get_Managed_VM_count(self.store)
            
        if vmCount > 0:
          iter = self.list_model.insert_before(None, None)
          self.list_model.set(iter,
                              self.SERVER_GROUP,'Uncategorized',
                              self.SERVER,'%4d' % len(nodes),
                              self.VMS,'%4d' % vmCount,
                              self.DUMMY,"")
          totalServerCount += len(nodes)
          totalVmCount +=vmCount

        # Iterate through managed nodes
        groups = manager.getGroupList()
        for i in groups.keys():
          nodes = groups[i].getNodeList()
          vmCount = 0
          for j in nodes.keys():
            vmCount += nodes[j].get_Managed_VM_count(self.store)    
          if vmCount > 0:
            iter = self.list_model.insert_before(None, None)
            self.list_model.set(iter,
                                self.SERVER_GROUP,str(i),
                                self.SERVER, '%4d' % len(nodes),
                                self.VMS,'%4d' % vmCount,
                                self.DUMMY, "")
            totalServerCount += len(nodes)
            totalVmCount += vmCount
            
        iter = self.list_model.insert_before(None, None)
        self.list_model.set(iter,
                            self.SERVER_GROUP,"Total",
                            self.SERVER,'%4d' % totalServerCount,
                            self.VMS,'%4d' % totalVmCount,
                            self.DUMMY,"")



#############
#    Util functions.. may move to util OR Managed Node
#    Also needs to cut this over to remote api
#



def createVBD(managed_node, filename, size=2048, backup=False):
    """Creates a file Virtual Block Device called 'name'
    at 'location'. 'size' is specified in kB.
    RAISES: IOError, OSError """

    if managed_node.node_proxy.file_exists(filename):
        if backup:
            managed_node.node_proxy.rename(filename,filename+'.bak')
        else:
            managed_node.node_proxy.remove(filename)

    # create the new disk block
    fd = managed_node.node_proxy.open(filename,'w')
    off = size * 1024L**2
    fd.seek(off, 0)
    fd.write('\x00')
    fd.close()




def cleanupQCDomain(managed_node, name, vbdlocation=''):
    """ Delete the xen configuration file and associated
    disk devices created during a quickcreate"""
    #domfilename = os.path.join(managed_node.config.get(XMConfig.PATHS,constants.prop_xenconf_dir),name)
    #if managed_node.node_proxy.file_exists(domfilename):
    #    managed_node.node_proxy.remove(domfilename)

    if name in managed_node.get_dom_names():
        dom = managed_node.get_dom(name)
        dom_config = None
        if dom:
            dom_config = dom.get_config()

        if dom_config:
            domfilename = dom_config.filename
            if domfilename and managed_node.node_proxy.file_exists(domfilename):
                managed_node.node_proxy.remove(domfilename)


            for file in dom_config.getDisks():
                # skip read only volumes, they are likely to be shared
                # or cdroms.
                # THIS IS NOT COMPLETE SOLN, NEED to prompt user for each
                # disk is the right soln
                if file.mode.find("w") == -1:
                    continue
                if file.type == 'phy':
                    try:
                        managed_node.lvm_proxy.removeLogicalVolume(file.filename)
                        print 'deleting: ' + file.filename
                    except OSError, err:
                        print "error deleting " + file.filename,err
                elif file.type in ['file','tap:aio','tap:qcow','tap:vmdk']:
                    if managed_node.node_proxy.file_exists(file.filename):
                        print 'deleting: ' + file.filename
                        managed_node.node_proxy.remove(file.filename)
    else:
        print "Couldn't find the domain %s. Skipping deletion" % name






########## UI Utils #################

class CBException:
    def __init__(self, context, ex):
        self.context = context
        self.ex = ex

def cb_showmsg(msg):
    try:
        gtk.gdk.threads_enter()
        showmsg(msg)
    finally:
        gtk.gdk.threads_leave()


def showmsg(msg, parent=None):
    """Displays the status message passed as an argument in a popup window"""
    input = msg

    d_msg = ""
    context = ""

    if isinstance(input, CBException):
        (context, input) =  (input.context, input.ex)

    if input and isinstance(input, Exception) and input.__dict__.get("args"):
        args = input.args
        if len(args) == 0:
            d_msg = str(input)
        else:
            for mx in args:
                if isinstance(mx, list):
                    for m in mx:
                        d_msg += str(m) + "\n"
                else:
                    d_msg += str(mx)
    else:
        d_msg = str(input)

    if context is not None and context != "":
        d_msg = context +"\n" + d_msg

    wtree.get_widget('StatusMsg').set_text(d_msg)
    dialog = wtree.get_widget('StatusDialog')
    if parent:
        dialog.set_transient_for(parent) 
    dialog.run()
    dialog.hide()


from threading import Thread
import threading
import convirt.core.utils.NodeProxy
class UIWorker(Thread):
    """

    """
    def __init__(self, ui_callback,context=None,
                 success_cb = None,failure_cb = cb_showmsg,
                 lock_ui = False):
                 #ui_cb_args=(), ui_cb_kwargs={}) :

        Thread.__init__(self)
        self.setDaemon(True)
        self.ui_callback = ui_callback
        #self.ui_cb_args = ui_cb_args
        #self.ui_cb_kwargs = ui_cb_kwargs
        self.success_cb = success_cb
        self.failure_cb = failure_cb
        self.context = context
        self.result = None
        self.done = False
        self.lock_ui = lock_ui




    def run(self):
        try:
            try:
                #self.ui_callback(*self.ui_cb_args,**self.ui_cb_kwargs)
                print "calling cb", self.context

                if not self.lock_ui:
                    self.result = self.ui_callback()
                else:
                    # might have to give up livelyness here..
                    # as we throw up login dialog from backend op too :(
                    try:
                        gtk.gdk.threads_enter()
                        self.result = self.ui_callback()
                    finally:
                        gtk.gdk.threads_leave()

                # call the success_cb
                if self.success_cb:
                    print "posting success cb", self.context
                    gobject.idle_add(self.success_cb)
            except Exception ,ex:
                if self.failure_cb:
                    #if self.context is not None :
                    #    msg = self.context + ":" + str(ex)
                    #else:
                    #    msg = str(ex)
                    traceback.print_exc()
                    print "posting exception cb", self.context, ex
                    gobject.idle_add(self.failure_cb, CBException(self.context, ex))

            if self.context is None:
                self.context = ""
            print "thread done. Context ", self.context
            self.done = True
        finally:
            # clean up andy local storage with the thread
            convirt.core.utils.NodeProxy.Node.clean_locals()

def show_wait_cursor(win = None):
    """ Wait cursor displayed / all input disabled """

    if win is None:
        win = main_context["main_window"]
    watch = gtk.gdk.Cursor(gtk.gdk.WATCH)
    gdkwin = win.window
    gdkwin.set_cursor(watch)
    gtk.gdk.flush()

def hide_wait_cursor(win = None):
    if win is None:
        win = main_context["main_window"]
    gdkwin = win.window
    gdkwin.set_cursor(None)




def confirmation(msg):
    """Displays a confirmation dialog message and returns a boolean
    representing the result"""

    wtree.get_widget('ConfirmationDialog').set_title("Confirm")
    wtree.get_widget('ConfirmationMsg').set_text(msg)
    dialog = wtree.get_widget('ConfirmationDialog')
    res = dialog.run() == gtk.RESPONSE_YES
    dialog.hide()
    return res

def file_selection(managed_node,
                   title, fstype,
                   init_folder=None, init_file=None,
                   parentwin=None, force_local = False,
                   multi_select = False):
    """This function will setup a file selection dialogue and return a tuple
    consisting of a boolean which is true if the user selected a filename,
    followed by the filename. fstype is either "open" or "save" or
    "select_folder" and title is whateveryou want displayed as the
    window's title bar. The init_folder and init_file params would be used
    as initial values for the dialog box"""

    gnome_vfs_enabled = True
    if main_context.has_key("client_config"):
        client_config = main_context["client_config"]
        gnome_vfs_prop_val = client_config.get(XMConfig.CLIENT_CONFIG,
                                              constants.prop_gnome_vfs_enabled)
        if gnome_vfs_prop_val is not None:
             gnome_vfs_enabled = eval(gnome_vfs_prop_val)

    if gnome_vfs_enabled:
        back_end = 'gnome-vfs'
    else:
        back_end = None

    if back_end is None and managed_node.is_remote() and not force_local:
        remote_dlg = RemoteFileDialog(wtree)
        remote_dlg.show(managed_node, title, fstype, init_folder, init_file,
                    parentwin, multi_select=multi_select)
        ret = remote_dlg.run()
        res = ret == gtk.RESPONSE_OK
        filename = remote_dlg.get_filename()
        if multi_select:
            file_list = []
            if filename and filename is not "":
                file_list = filename.split(",")
                return (res, file_list)
        else:
            return (res, filename)


    if fstype == "save":
        filesel = gtk.FileChooserDialog(title,
                                        action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_SAVE,
                                                 gtk.RESPONSE_OK),
                                        backend=back_end)
    elif fstype == "open":
        filesel = gtk.FileChooserDialog(title,
                                        action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_OPEN,
                                                 gtk.RESPONSE_OK),
                                        backend = back_end)
        if multi_select:
            filesel.set_select_multiple(True)
    elif fstype == "select_folder":
        filesel = gtk.FileChooserDialog(title,
                                        action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_OPEN,
                                                 gtk.RESPONSE_OK),
                                        backend= back_end)

    # enable remote file browsing.
    filesel.set_local_only(False)


    # if remote node, create a url and set it.
    if managed_node.is_remote() and not force_local:
        hostname = managed_node.hostname
        username = managed_node.username
        if username is None:
            username = os.getlogin()
        url = "ssh://"+username+"@"+hostname+"/"
        title = filesel.get_title()
        filesel.set_title(title + ":" + username +"@"+ hostname)
        if init_folder is not None and init_folder != "":
            if init_folder[0] == '/':
                url = url + init_folder[1:]
                filesel.set_current_folder_uri(url)
        else:
            filesel.set_current_folder_uri(url)
    else: # local node
        if init_folder is not None and init_folder is not "":
            filesel.set_current_folder(init_folder)

    if init_file is not None:
        filesel.set_current_name(init_file)

    if parentwin:
        filesel.set_transient_for(parentwin)
        filesel.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

    res = filesel.run() == gtk.RESPONSE_OK

    filelist = []
    if res:
        uris = filesel.get_uris()
        if managed_node.is_remote() and not force_local:
            selected_uri = uris[0]
            selected_uri = filesel.get_uri()
            selected_uri = urllib.unquote(selected_uri)
            if selected_uri and selected_uri.find(hostname) == -1:
                msg = "File not selected from the managed node.\n"
                msg += "Please use Open Location and use ssh://"+username+"@" +hostname +"/ to access remote files.\n"
                msg += "Open Location available through right click menu."
                showmsg(msg)
                filelist = []
                filesel.destroy()
                return (None, filelist)

        for u in uris:
            filelist.append(get_filename_from_uri(u))


    filesel.destroy()

    if multi_select:
        return (res, filelist)
    else:
        if len(filelist) > 0:
            return (res, filelist[0])
        else:
            return (res, None)

def get_filename_from_uri(uri):
    if uri == None:
        return None
    # Kludge : as urllib functions require http
    if uri.find("ssh") == 0:
        uri =  uri.replace("ssh", "http", 1)
    (prot, host, path,param,query,fragments ) = urlparse.urlparse(uri)
    if path is not None:
        return path


# Cell renderers for the colums in the tree view.
def get_state_pixbuf(column, renderer, model, iter, appliance_store=None):
    """ Call back to display the state column image"""
    dom_name = model.get_value(iter,0)
    node_type = model.get_value(iter,1)
    managed_node =  model.get_value(iter,2)
    g_name = model.get_value(iter, 3)
    provider_id = model.get_value(iter, 4)
    
    if node_type is None :
        return

    pb = unknown_pb

    if node_type == MANAGED_NODE:
        pb = node_pb
    elif node_type == IMAGE_STORE:
        pb = dc_pb
    elif node_type == IMAGE_GROUP:
        pb = image_store_pb
    elif node_type == IMAGE:
        pb = image_pb
        if g_name:
            #provider_id = g_name
            ppb = None
            if provider_id:
                ppb = ApplianceLogos.get_provider_logo(appliance_store,
                                                       provider_id)
            if ppb:
                pb = ppb

    elif managed_node is not None and node_type == DOMAIN: # dom
        dom = managed_node.get_dom(dom_name)

        if dom and dom.is_resident():
            try:
                state = dom.get_state()
            except xmlrpclib.Fault :
                state = None

            if state is not None and state == VM.PAUSED:
                pb = paused_pb
            else:
                pb = resident_pb
        else:
            pb = not_resident_pb
    elif node_type == SERVER_POOL:
        pb = pool_pb
    elif node_type == DATA_CENTER:
        pb = dc_pb


    renderer.set_property('pixbuf', pb)

def get_node_type_pb(type):
    if type == SERVER_POOL:
        return pool_pb
    elif type == MANAGED_NODE:
        return node_pb

### create a node list

def populate_nodes(manager, view, model, dummy = True, platform = None):
    """
    refresh the nodes.
    Not expected to be called other than constructor.

    """
    model.clear()

    # Append a Global Server Pool
    d_iter = model.append(None,
                          ["Data Center",
                           DATA_CENTER,
                           None,
                           None,
                           None,
                           None])

    # sort them
    names = manager.getNodeNames()
    name_list = []
    for name in names:
        if platform:
            n = manager.getNode(name)
            if n.get_platform() != platform:
                continue
        name_list.append(name)
    name_list.sort()

    # Kludge, append a blank node so the expanders show up
    nodes = manager.getNodeList()
    for name in name_list:
        node = nodes[name]
        iter = \
             model.append(d_iter,
                          [node.hostname,
                           MANAGED_NODE,
                           node,
                           None,
                           None,
                           None])
        if dummy:
            append_dummy_row(model, iter)

    # add the server pools
    group_names = manager.getGroupNames()
    group_names.sort()

    groups = manager.getGroupList()
    for group_name in group_names:
        group = groups[group_name]
        g_iter = model.append(d_iter,
                              [group.name,
                               SERVER_POOL,
                               group,
                               None,
                               None,
                               None])

        # sort them
        names = group.getNodeNames()
        name_list = []
        for name in names:
            if platform:
                n = group.getNode(name)
                if n.get_platform() != platform:
                    continue
            name_list.append(name)
        name_list.sort()

        # Kludge, append a blank node so the expanders show up
        nodes = group.getNodeList()
        for name in name_list:
            node = nodes[name]
            iter = \
                 model.append(g_iter,
                              [node.hostname,
                               MANAGED_NODE,
                               node,
                               group.name,
                               None,
                               None])
            if dummy:
                append_dummy_row(model, iter)


        # NOTE : if this is re-introduced, we need to change
        # UI code for adding and removing groups

        #if len(names) == 0:
        #    if dummy:
        #        append_dummy_row(model, g_iter, "Empty")

    view.expand_row((0,),False)
    view.set_cursor((0,))

def append_dummy_row( model, iter, msg = "VMs not found"):
    model.append(iter, [msg,
                        "DUMMY",None, None, None, None])


# moved to util to prevent cirular dependency.
class ApplianceLogos:
    logos = {}

    @classmethod
    def get_provider_logo(cls, appliance_store, provider_id):
        if not cls.logos.get(provider_id):
            if not provider_id:
                return appliance_pb

            cache_dir = os.path.join(appliance_store.get_cache_dir(),
                                     provider_id)
            if not os.path.exists(cache_dir):
                os.makedirs(cache_dir)
            for ext in (".gif", ".ico", ".png"):
                logo_path =  os.path.join(cache_dir,  provider_id + ext)
                try:
                    pb = gtk.gdk.pixbuf_new_from_file(logo_path)
                    if pb:
                        cls.logos[provider_id] = pb.scale_simple(16,16,
                                                                 gtk.gdk.INTERP_BILINEAR )
                        break
                except Exception, ex:
                    pass
                if not cls.logos.get(provider_id):
                    try:
                        print "provider_id ", provider_id
                        # fetch the logo from the given url
                        logo_url = appliance_store.get_logo_url(provider_id)
                        if logo_url is not None:
                            p = urlparse.urlparse(logo_url)[2]
                            logo_name= os.path.basename(p)
                            frag = logo_name.split(".")
                            if len(frag) > 1:
                                ext = "." + frag[-1]
                            else:
                                ext = ".gif" # can do better with content-type :TBD
                            logo_file = os.path.join(cache_dir,
                                                     provider_id + ext)
                            fetch_isp(logo_url,logo_file, "image")
                            pb = gtk.gdk.pixbuf_new_from_file(logo_file)
                            if pb:
                                cls.logos[provider_id] = pb.scale_simple(16,16,
                                                                         gtk.gdk.INTERP_BILINEAR )
                    except Exception, e:
                        print "Exception processing ", provider_id, e
                        traceback.print_exc()
                        pass

                # set it to none so we do not keep doing scans
                if not cls.logos.get(provider_id):
                    cls.logos[provider_id] = appliance_pb
                    print "setting default for ", provider_id

        return cls.logos[provider_id]

from ui_utils import init_combo, set_value, get_value
class SelectPlatformDlg:

    def __init__(self):

        """ Constructor"""
        self.gladefile = gladefile
        self.wtree = gtk.glade.XML(self.gladefile, "SelectPlatformDlg")
        self.dialog = self.wtree.get_widget("SelectPlatformDlg")

        self.platform_combo = self.wtree.get_widget("platform_combo")
        self.dialog.set_default_response(gtk.RESPONSE_OK)
        # connect the handlers
        self.wtree.signal_connect("on_select_platform_cancel_clicked",
                                  self.on_cancel_clicked)
        self.wtree.signal_connect("on_select_platform_ok_clicked",
                                  self.on_ok_clicked)

        self.registry = None
        self.parentwin = None
        self.selected_platform = None

    def show(self, registry, parentwin = None):

        self.registry = registry
        self.parentwin = parentwin
        self.selected_platform = None
        
        if parentwin:
            self.dialog.set_transient_for(parentwin)
            self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

            
        map = {}
        for platform, info in registry.get_platforms().iteritems():
            name = info["name"]
            map[name] = platform
            
        init_combo(self.platform_combo, map)
        self.dialog.show()

        ret = self.dialog.run()
        self.dialog.destroy()
        if ret == gtk.RESPONSE_OK:
            return True
        else:
            return False

    def on_cancel_clicked(self, widget):
        return False


    def on_ok_clicked(self, widget):
        self.selected_platform = get_value(self.platform_combo)
        return True

def update_config_props(config, model, excluded = []):
    # take misc model remove deleted props
    delete_props = []
    if config is None:
        return
    for prop in config:
        if prop not in excluded:
            # check if it exists in the misc model
            found = False
            iter = model.get_iter_first()
            while iter:
                key = model.get_value(iter, 0)
                if key == prop:
                    found = True
                    break
                iter = model.iter_next(iter)

            if not found:
                delete_props.append(prop)

    for prop in delete_props:
        del config.options[prop]

    # now copy update values from model to the config.
    iter = model.get_iter_first()
    while iter:
        key = model.get_value(iter, 0)
        value = model.get_value(iter,1)
        # We got the string representation, guess and make it proper value
        key = key.strip()
        value = value.strip()
        value = guess_value(value)
        if key is not None and key is not '':
            config[key] = value
        iter = model.iter_next(iter)

    #config.dump()




### Module initialization
# Types of nodes in the tree
SERVER_POOL = "SERVER_POOL"
MANAGED_NODE = "MANAGED_NODE"
DOMAIN  = "DOMAIN"
DATA_CENTER= "DATA_CENTER"

IMAGE_STORE = "IMAGE_STORE"
IMAGE = "IMAGE"
IMAGE_GROUP = "IMAGE_GROUP"


(path, gladefile) = get_path("convirt.glade")
if not path:
    print "ERROR: Couldn't find glade interface definition file!"
    sys.exit(1)   # bad, but ok for now.


paused_pb   = gtk.gdk.pixbuf_new_from_file(path +\
                                           "/pixmaps/small_pause.png")
resident_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                           "/pixmaps/small_started_state.png")
not_resident_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                               "/pixmaps/small_shutdown.png")

node_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                       "/pixmaps/small_node.png")

dc_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                     "/pixmaps/small_pool.png")
pool_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                       "/pixmaps/group.png")

unknown_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                          "/pixmaps/small_unknown_state.png")

image_store_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                          "/pixmaps/small_image_store.png")

image_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                        "/pixmaps/small_image.png")

connected_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                        "/pixmaps/small_connect_blue.png")
disconnected_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                        "/pixmaps/small_disconnect_yellow.png")

appliance_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                        "/pixmaps/small_appliance.png")

convirt_pb = gtk.gdk.pixbuf_new_from_file(path + \
                                         "/pixmaps/convirt.png")

## error_img = gtk.Image()
## error_img.set_from_stock(gtk.STOCK_DIALOG_ERROR,
##                          gtk.ICON_SIZE_MENU)
## error_pb = error_img.get_pixbuf()

## warn_img = gtk.Image()
## warn_img.set_from_stock(gtk.STOCK_DIALOG_WARNING,
##                         gtk.ICON_SIZE_MENU)

## warn_pb = warn_img.get_pixbuf()

warn_pb = connected_pb
error_pb = disconnected_pb

gtk.window_set_default_icon(convirt_pb)

wtree = gtk.glade.XML(gladefile)

# KLUDGE : way to share global context
main_context = {}

file_editor =  FileViewEditDialog(wtree)
####
if __name__ == '__main__':
##     helper = CredentialsHelperDialog(wtree)
##     cred = helper.get_credentials("foobar", "root")
##     if cred:
##         print cred.get_username(), cred.get_password()
    dlg = MigrationChecksResultsDlg()
    err_list = (("Mem", "Test"), ("CPU", "TEST"))
    warn_list = (("HVM", "Warn Test"), ("Platform", "Warn TEST 1 sdslfjsdlfjsdjfs sdsdfsd sdfvdsfsdfs \n Second Line"))
    ret = dlg.show(err_list, warn_list)
    print ret
